from pathlib import Path
from pkgutil import extend_path
from single_source import get_version
from typing import List, Optional

__path__: List[str] = extend_path(__path__, __name__)
__version__: Optional[str] = get_version(__name__, Path(__file__).parent.parent)



from typing import Callable
from typing import Dict
from typing import Optional
from typing import Tuple

from cleo.exceptions import LogicException
from cleo.loaders.factory_command_loader import FactoryCommandLoader


class CommandLoader(FactoryCommandLoader):
    def __init__(self, factories: Dict[str, Callable] = {}) -> None:
        super().__init__(factories)

    def resolve(
        self,
        command_class: str,
        module: Optional[str] = None,
        package: Optional[str] = None,
    ) -> Tuple[str, str, Optional[str]]:
        if module is None:  # command_class is an FQN
            fqn_parts = command_class.split(".")
            module = ".".join(fqn_parts[:-1])  # all but last part
            command_class = fqn_parts[-1]  # last part

        return command_class, module, package

    def register_factory(self, command_name: str, factory: Callable) -> None:
        if command_name in self._factories:
            raise LogicException(f'The command "{command_name}" already exists.')

        self._factories[command_name] = factory



from importlib import import_module
from typing import TYPE_CHECKING
from typing import Callable
from typing import Optional
from typing import Type

from cleo.application import Application as CleoApplication

from bibscrap import __version__
from bibscrap.cli.commands import CommandLoader
from bibscrap.cli.commands.command import Command
from bibscrap.util import get_optional

BUILTIN_COMMANDS_MODULE = "bibscrap.cli.commands"

BUILTIN_COMMANDS = [
    "about",
]


class Application(CleoApplication):
    """Bibscrap application class."""

    def __init__(self) -> None:
        super().__init__("bibscrap", get_optional(__version__))
        self.set_command_loader(CommandLoader())
        for builtin_command in BUILTIN_COMMANDS:
            self.register_builtin_command(builtin_command)

    def register_command(
        self,
        command: str,
        command_class: str,
        module: Optional[str] = None,
        package: Optional[str] = None,
    ) -> None:
        """Register a command with the command loader.

        Args:
            command: The command to register.
            command_class: The command's :py:class:`Command` class name.
            module: The module containing the command's :py:class:`Command` class. If
                :py:const:`None`, then `command_class` must be a fully qualified name.
            package: Used to anchor the import when `module` is specified in relative terms.
        """
        command_class, module, package = self.command_loader.resolve(
            command_class, module, package
        )

        def _deferred() -> Callable:
            imported_module = import_module(module, package)
            imported_command_class = getattr(imported_module, command_class)
            return imported_command_class()

        self.command_loader.register_factory(command, _deferred)

    def register_builtin_command(self, command: str) -> None:
        """Register a builtin Bibscrap command.

        Args:
            command: The builtin command to register.
        """
        parts = command.split(" ")
        submodule = ".".join(parts)
        command_class = "{}Command".format("".join(map(str.title, parts)))
        fqn = f"{BUILTIN_COMMANDS_MODULE}.{submodule}.{command_class}"
        self.register_command(command, fqn)

    @property
    def command_loader(self) -> CommandLoader:
        return self._command_loader


def main() -> int:
    return Application().run()


if __name__ == "__main__":
    main()



from .command import Command


class AboutCommand(Command):

    name = "about"

    description = "Shows information about Bibscrap."

    def handle(self) -> None:
        self.info(
            "Bibscrap - Semi-automated Tools for Systematic Literature Reviews</info>"
        )
        self.line_blank()
        self.comment(
            "Bibscrap provides tools that help researchers prepare rigorous systematic "
            "literature reviews."
        )
        self.line_blank()
        self.comment(
            "See <fg=blue>https://github.com/cotterell/bibscrap/</fg> for more information."
        )



from typing import Callable
from typing import Optional
from typing import Type
from typing import TypeVar


T = TypeVar("T")


def get_optional(arg: Optional[T]) -> T:
    """Return the value for `arg` if not :py:const:`None`, else raise an :py:exc:`AssertionError`.

    Args:
        arg: The :py:class:`Optional`.

    Returns:
        The value held by `arg`.

    Raises:
        AssertionError: If there is no value present in `arg` (i.e., ``arg is None``).

    References:
        `@gvanrossum <https://github.com/gvanrossum>`__ Jun 11, 2019. Comment in
        `python/typing#645 <https://github.com/python/typing/issues/645#issuecomment-501057220>`__.
    """
    assert arg is not None
    return arg



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['bibscrap', 'bibscrap.cli', 'bibscrap.cli.commands']

package_data = \
{'': ['*']}

install_requires = \
['cleo>=1.0.0a4,<2.0.0', 'scrapy>=2.5.1,<3.0.0', 'single-source>=0.2.0,<0.3.0']

extras_require = \
{'dev': ['black>=21.9b0,<22.0',
         'pre-commit>=2.15.0,<3.0.0',
         'tox>=4.0.0a9,<5.0.0',
         'darglint>=1.8.0,<2.0.0',
         'pytest>=6.2.5,<7.0.0',
         'coverage>=6.0.2,<7.0.0',
         'codecov>=2.1.12,<3.0.0'],
 'docs': ['Sphinx>=4.2.0,<5.0.0',
          'sphinx-rtd-theme>=1.0.0,<2.0.0',
          'sphinxcontrib-bibtex>=2.4.1,<3.0.0']}

entry_points = \
{'console_scripts': ['bibscrap = bibscrap.cli.app:main']}

setup_kwargs = {
    'name': 'bibscrap',
    'version': '0.1.0a1',
    'description': 'Semi-automated tools for systematic literature reviews.',
    'long_description': "========\nbibscrap\n========\n\n.. image:: https://img.shields.io/pypi/v/bibscrap?style=flat\n   :target: https://pypi.org/project/bibscrap/\n   :alt: PyPI version\n\n.. image:: https://app.travis-ci.com/cotterell/bibscrap.svg\n   :target: https://app.travis-ci.com/cotterell/bibscrap\n\n.. image:: https://readthedocs.org/projects/bibscrap/badge/?version=latest\n   :target: https://bibscrap.readthedocs.io/en/latest/\n   :alt: bibscrap's documentation\n\n\n.. image:: https://codecov.io/gh/cotterell/bibscrap/branch/main/graph/badge.svg?token=TQQWS0OQ0E\n   :target: https://codecov.io/gh/cotterell/bibscrap\n   :alt: Code coverage\n\n.. image:: https://img.shields.io/pypi/l/bibscrap.svg\n   :target: https://github.com/cotterell/bibscrap/blob/master/LICENSE.rst\n   :alt: MIT license\n\n.. image:: https://img.shields.io/badge/code%20style-black-161b22.svg\n   :target: https://github.com/psf/black\n   :alt: Code style: black\n\nOverview\n========\n\nThe **bibscrap** package provides semi-automated tools for systematic literature reviews.\n\nRequirements\n============\n\n* Python 3.9+\n\nInstall\n=======\n\nTo install Bibscrap using **pipenv**::\n\n  $ pipenv install --pre bibscrap\n\nTo install Bibscrap using **pip**::\n\n  $ pip install --pre bibscrap\n\nDocumentation\n=============\n\nDocumentation is available online at https://bibscrap.readthedocs.io/ and in the\n``docs`` directory.\n\nContributors: Getting Started\n=============================\n\nTo download the development version of **bibscrap**, follow the instructions\nprovided below::\n\n  $ git clone https://github.com/cotterell/bibscrap.git\n  $ cd bibscrap\n  $ poetry install -v\n\nIf you are part of the **bibscrap** development team, then you should also\ninstall the development packages::\n\n  $ pre-commit install\n\nContributors\n============\n\n=====================  ==========================================================  ============\nContributor            GitHub                                                      Role\n=====================  ==========================================================  ============\nAidan Killer           `@aikill <https://github.com/aikill>`_                      Developer\nJack Solomon           `@jbs26156 <https://github.com/jbs26156>`_                  Developer\nMatthew Pooser         `@mpooser <https://github.com/mpooser>`_                    Developer\nMichael E. Cotterell   `@mepcotterell <https://github.com/mepcotterell>`_          Maintainer\nMitchell Casleton      `@MitchellCasleton <https://github.com/MitchellCasleton>`_  Developer\nMy Nguyen              `@mynguyen0628 <https://github.com/mynguyen0628>`_          Developer\n=====================  ==========================================================  ============\n",
    'author': 'Michael E. Cotterell',
    'author_email': 'mepcotterell@gmail.com',
    'maintainer': 'Michael E. Cotterell',
    'maintainer_email': 'mepcotterell@gmail.com',
    'url': 'https://github.com/cotterell/bibscrap',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'extras_require': extras_require,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



from __future__ import annotations

from cleo.commands.command import Command as BaseCommand


class Command(BaseCommand):
    """Bibscrap command class."""

    def line_blank(self) -> None:
        """Write a blank line as output."""
        self.line("")
